# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "25/03/2019"


import os
import shutil
import tempfile
import unittest

from orangecontrib.tomwer.test.utils import OrangeWorflowTest

from tomwer.core.utils.scanutils import MockEDF
from tomwer.synctools.ftseries import _QDKRFRP, _QAxisRP, QReconsParams
from tomwer.core.scan.scanbase import TomwerScanBase
from tomwer.core.process.reconstruction.darkref.params import Method as DkrfMethod
from tomwer.core.scan.scanfactory import ScanFactory
from tomwer.test.utils import skip_gui_test, skip_orange_workflows_test
import time

import logging
import logging

logging.disable(logging.INFO)

_logger = logging.getLogger(__name__)


@unittest.skipIf(skip_orange_workflows_test(), reason="skip orange workflows tests")
@unittest.skipIf(skip_gui_test(), reason="skip gui test")
@unittest.skip("Fail on CI...")
class TestReconstructionWidgets(OrangeWorflowTest):
    """Test behavior for several scans with the following scheme

        * DataSelector
        * DarkRefs orange widget (reconstruct both dark and flat)
        * Ftaxis orange widget
        * Ftseries orange widget

    Then process the workflow and check if the different values of ReconsParams
    are valid. And that in all case, pass all processes
    """

    @classmethod
    def setUpClass(cls):
        OrangeWorflowTest.setUpClass()
        # create widgets
        cls.dataSelectorNode = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataSelectorOW.DataSelectorOW"
        )
        cls.darkRefNode = cls.addWidget(
            cls,
            "orangecontrib.tomwer.widgets.reconstruction.DarkRefAndCopyOW.DarkRefAndCopyOW",
        )
        cls.ftAxisNode = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.reconstruction.AxisOW.AxisOW"
        )
        cls.ftSeriesNode = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.reconstruction.FtseriesOW.FtseriesOW"
        )
        cls.processOrangeEvents(cls)

        cls.link(cls, cls.dataSelectorNode, "data", cls.darkRefNode, "data")
        cls.link(cls, cls.darkRefNode, "data", cls.ftAxisNode, "data")
        cls.link(cls, cls.ftAxisNode, "data", cls.ftSeriesNode, "data")
        cls.processOrangeEvents(cls)

        cls.dataSelectorWidget = cls.getWidgetForNode(cls, cls.dataSelectorNode)
        cls.darkRefWidget = cls.getWidgetForNode(cls, cls.darkRefNode)
        cls.axisWidget = cls.getWidgetForNode(cls, cls.ftAxisNode)
        cls.ftSeriesWidget = cls.getWidgetForNode(cls, cls.ftSeriesNode)

        # Set we only want to simulate the reconstruction
        cls.ftSeriesWidget._ftserie.reconsStack.setMockMode(True)
        cls.ftSeriesWidget._ftserie.setForceSync(True)
        cls.darkRefWidget.setForceSync(True)
        cls.axisWidget.setLocked(True)

    @classmethod
    def tearDownClass(cls):
        for node in (
            cls.dataSelectorNode,
            cls.darkRefNode,
            cls.ftAxisNode,
            cls.ftSeriesNode,
        ):
            cls.removeNode(cls, node)
        _logger.info("all nodes removed")
        cls.dataSelectorWidget = None
        cls.darkRefWidget = None
        cls.axisWidget = None
        cls.ftSeriesWidget = None
        OrangeWorflowTest.tearDownClass()

    def setUp(self):
        OrangeWorflowTest.setUp(self)
        self._source_dir = tempfile.mkdtemp()

        def create_scan(folder_name):
            _dir = os.path.join(self._source_dir, folder_name)
            MockEDF.mockScan(scanID=_dir, nRadio=10, scan_range=180, n_extra_radio=2)
            return ScanFactory.create_scan_object(_dir)

        # tune scans
        # scan_1 has no reconstruction parameters
        self.scan_1 = create_scan("scan_1")
        # scan_2 has only dark_ref reconstructions parameters
        self.scan_2 = create_scan("scan_2")
        self.scan_2.ftseries_recons_params = QReconsParams(empty=True)
        self.scan_2.ftseries_recons_params.dkrf = _QDKRFRP()
        self.scan_2.ftseries_recons_params.dkrf.ref_calc_method = "None"
        # scan_3 has only ftaxis reconstructions parameters
        self.scan_3 = create_scan("scan_3")
        self.scan_3.ftseries_recons_params = QReconsParams(empty=False)
        self.scan_3.ftseries_recons_params.copy(self.scan_2.ftseries_recons_params.dkrf)
        # scan_4 has default reconstruction parameters
        self.scan_4 = create_scan("scan_4")
        self.scan_4.ftseries_recons_params = QReconsParams()
        self.scan_4.ftseries_recons_params.axis = _QAxisRP()
        self.scan_4.ftseries_recons_params.axis.to_the_center = False

        # tune processes
        self.darkRefWidget.recons_params.ref_calc_method = "Median"
        self.darkRefWidget.recons_params._set_skip_if_exist(False)
        self.axisWidget.recons_params.to_the_center = False
        self.axisWidget.recons_params.plot_figure = False

        # deactivate dark and ref copy
        self.darkRefWidget.widget._refCopyWidget.setChecked(False)

        self.ftSeriesWidget.recons_params.ref_calc_method = "Average"
        self.ftSeriesWidget.recons_params.dkrf._set_skip_if_exist(True)
        self.ftSeriesWidget.recons_params.plot_figure = True
        self.ftSeriesWidget.recons_params.to_the_center = True

        self._scans = (self.scan_1, self.scan_2, self.scan_3, self.scan_4)
        for _scan in self._scans:
            self.dataSelectorWidget.addScan(_scan)
        self.app.processEvents()

        # skip axis pop up
        self.axisWidget._skip_exec(True)

    def tearDown(self):
        shutil.rmtree(self._source_dir)
        OrangeWorflowTest.tearDown(self)

    def testProcessing(self):
        """Test the different values of reconstruction parameters during the
        workflow execution"""
        # validate all scans
        for _scan in self._scans:
            self.app.processEvents()
            assert isinstance(_scan, TomwerScanBase)
            self.dataSelectorWidget.setActiveScan(_scan)

            # emit scan to the be managed by darkref
            self.dataSelectorWidget.widget._selectActiveScan()
            self.processOrangeEventsStack()

            # here we are in darkrefs. Recons params should be copy from darkref
            # to scan.ftseries_recons_params
            self.assertTrue(
                self.darkRefWidget.recons_params.ref_calc_method is DkrfMethod.median
            )
            self.assertTrue(
                _scan.ftseries_recons_params.dkrf.ref_calc_method is DkrfMethod.median
            )
            self.assertTrue(self.darkRefWidget.recons_params.overwrite_ref is True)
            self.assertTrue(_scan.ftseries_recons_params.dkrf.overwrite_ref is True)
            self.assertTrue(isinstance(_scan.ftseries_recons_params, QReconsParams))
            self.assertTrue(isinstance(_scan.ftseries_recons_params.dkrf, _QDKRFRP))

            for t in range(0, 5):
                time.sleep(0.1)
                self.processOrangeEventsStack()
                self.app.processEvents()

            self.assertTrue(isinstance(_scan.ftseries_recons_params, QReconsParams))

            # here we are in ftaxis. Axis parameters should also be copied from
            # AxisOW.recons_params to TomoBase.ftseries_recons_params.axis
            self.assertTrue(self.axisWidget.recons_params.plot_figure is False)
            self.assertTrue(isinstance(_scan.ftseries_recons_params, QReconsParams))
            self.processOrangeEventsStack()

            # here we are in ftseries. ftseries copy parameters from axis and
            # dkrf if any.
            self.assertTrue(
                self.ftSeriesWidget.recons_params.dkrf.ref_calc_method
                is DkrfMethod.median
            )
            self.assertTrue(
                _scan.ftseries_recons_params.dkrf.ref_calc_method is DkrfMethod.median
            )
            self.assertTrue(
                self.ftSeriesWidget.recons_params.dkrf.overwrite_ref is True
            )
            self.assertTrue(_scan.ftseries_recons_params.dkrf.overwrite_ref is True)
            self.app.processEvents()

        while self.app.hasPendingEvents():
            self.app.processEvents()


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestReconstructionWidgets,):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sys
import argparse
from pypushflow.representation.scheme.ows_parser import OwsParser
from tomwer.core.scan.scanfactory import ScanFactory
from pypushflow.Workflow import ProcessableWorkflow
from tomwer.core.scan.blissscan import BlissScan
import signal

logging.basicConfig(level=logging.DEBUG)
_logger = logging.getLogger(__name__)


def _exec(scheme, scan=None, timeout=None):
    """

    :param scheme:
    :param scan:
    :param timeout:
    :return:
    """
    # set up workflow
    if len(scheme.start_nodes()) is 0:
        _logger.warning("no start nodes found. Enable to process")
        return None
    mess = " ".join(
        (
            "start processing of scheme '{}'".format(scheme.title or "unknow"),
            "with",
            str(scan),
        )
    )
    _logger.info(mess)
    workflow = ProcessableWorkflow(scheme=scheme)

    # add SIGINT capture
    def signal_handler(sig, frame):
        _logger.warning("stop workflow execution on user request")
        workflow._end_actor.join(0)
        sys.exit(0)

    signal.signal(signal.SIGINT, signal_handler)

    workflow._start_actor.trigger(("data", scan.to_dict()))
    workflow._end_actor.join(timeout)
    res = workflow._end_actor.out_data
    title = scheme.title or "unknow"
    _logger.info("workflow '{}' completed with {}".format(title, str(scan)))
    return res


def main(argv):
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "workflow_file",
        help="Path to the .ows file defining the workflow to process with the"
        "provided scan",
    )
    parser.add_argument(
        "scan_path",
        help="Path to data to be processes (master file if come from an hdf5 "
        "acquisition or EDF files folder if come from an EDF acquisition)",
    )
    parser.add_argument(
        "--entry", default=None, help="An entry should be specify for hdf5 files"
    )
    parser.add_argument(
        "--timeout", default=None, help="Timeout for the workflow execution"
    )
    parser.add_argument(
        "--debug",
        dest="debug",
        action="store_true",
        default=False,
        help="Set logging system in debug mode",
    )
    options = parser.parse_args(argv[1:])
    if options.entry is not None:
        scan = ScanFactory.create_scan_object(
            options.scan_path, entry=options.entry, accept_bliss_scan=True
        )
        scans = (scan,)
    else:
        scans = ScanFactory.create_scan_objects(
            options.scan_path, accept_bliss_scan=True
        )
    if len(scans) > 1:
        _logger.info("More than one scan found. Will process every scans")

    # tune the log level
    log_level = logging.INFO
    if options.debug is True:
        log_level = logging.DEBUG

    for log_ in ("tomwer", "pypushflow"):
        logging.getLogger(log_).setLevel(log_level)

    scheme = OwsParser.scheme_load(options.workflow_file, load_handlers=True)
    for scan in scans:
        _exec(scheme=scheme, scan=scan, timeout=options.timeout)


if __name__ == "__main__":
    main(sys.argv)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "15/06/2017"

from silx.gui import qt


class OctaveVersionGetter(qt.QGroupBox):
    """Simple class to set the octave version"""

    # bad hack : should be a boolean and not a string. Because we are looking
    # only if bellow 3.8 or not
    valueHasChanged = qt.Signal(float)

    def __init__(self, title, parent=None):
        """

        :param str title: the name of the groupbox
        :param QObject parent: the QObject parent of the GroupBox
        """
        qt.QGroupBox.__init__(self, title=title, parent=parent)
        self.setLayout(qt.QHBoxLayout())

        self._qrbInf3_8 = qt.QRadioButton("version < 3.8", parent=self)
        self._qrbInf3_8.setChecked(True)
        self.layout().addWidget(self._qrbInf3_8)
        self._qrbInf3_8.toggled.connect(self.valueChanged)
        self._qrbSup3_8 = qt.QRadioButton("version >= 3.8", parent=self)
        self.layout().addWidget(self._qrbSup3_8)
        self._qrbSup3_8.toggled.connect(self.valueChanged)

    def setVersionInf3_8(self, b):
        """

        :param boolean b: if true then target the octave version < 3.8
        """
        if b:
            self._qrbInf3_8.setChecked(True)
        else:
            self._qrbSup3_8.setChecked(True)

    def isVersionInf3_8(self):
        """
        :return: True if the octave version we are trying to load is oldest
            than 3.8
        """
        return self._qrbInf3_8.isChecked()

    def valueChanged(self):
        self.valueHasChanged.emit(3.6 if self.isVersionInf3_8() else 3.8)



# coding: utf-8
# /*##########################################################################
# Copyright (C) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "22/01/2017"


import unittest

from silx.gui import qt

from tomwer.core.process.reconstruction.darkref.params import Method as DkrfMethod
from tomwer.gui.reconstruction.ftserie import FtserieWidget
from tomwer.gui.reconstruction.darkref.darkrefcopywidget import DarkRefAndCopyWidget
from tomwer.synctools.ftseries import QReconsParams
from tomwer.test.utils import skip_gui_test


@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestReconsParamFtSerieDarkRef(unittest.TestCase):
    """Test that the FtSerie and Dark Ref instances are synchronized
    both with the ReconsParam instance"""

    def setUp(self):
        self.recons_params = QReconsParams()
        self._ftserie = FtserieWidget(parent=None, recons_params=self.recons_params)
        self._darkRefCopy = DarkRefAndCopyWidget(
            parent=None, reconsparams=self.recons_params
        )
        self.qapp = qt.QApplication.instance() or qt.QApplication([])

    def tearDown(self):
        self._ftserie.setAttribute(qt.Qt.WA_DeleteOnClose)
        self._darkRefCopy.setAttribute(qt.Qt.WA_DeleteOnClose)

    def testSync(self):
        """Make sure that connection between dkrf and ftserie are made.
        Some individual test (ftserie and ReconsParams) are also existing"""
        self._ftserie.show()
        self._darkRefCopy.show()
        ftserieEditor = self._ftserie.getReconsParamSetEditor()
        ftserieDKRf = ftserieEditor._dkRefWidget
        dkrfSkip = self._darkRefCopy.mainWidget.tabGeneral._skipOptionCB

        # test skipping option
        skipping = ftserieDKRf._qcbSkipRef.isChecked()
        self.assertTrue(dkrfSkip.isChecked() is skipping)
        self.assertTrue(self._ftserie.recons_params.dkrf["REFSOVE"] is not skipping)
        self.assertTrue(self._ftserie.recons_params.dkrf["DARKOVE"] is not skipping)
        ftserieDKRf._qcbSkipRef.setChecked(not skipping)
        self.qapp.processEvents()
        self.assertTrue(dkrfSkip.isChecked() is not skipping)
        self.assertTrue(self._ftserie.recons_params.dkrf["REFSOVE"] is skipping)
        self.assertTrue(self._ftserie.recons_params.dkrf["DARKOVE"] is skipping)

        # test dark pattern
        ftserieRef = ftserieEditor._dkRefWidget._qleDKPattern
        dkrfRef = self._darkRefCopy.mainWidget.tabExpert._darkLE
        self.assertTrue(dkrfRef.text() == ftserieRef.text())
        self.assertTrue(dkrfRef.text() == self._ftserie.recons_params.dkrf["DKFILE"])
        txt = dkrfRef.text()
        dkrfRef.setText(txt + "toto")
        dkrfRef.editingFinished.emit()
        self.qapp.processEvents()
        self.assertTrue(self.recons_params.dkrf["DKFILE"] == txt + "toto")
        self.assertTrue(dkrfRef.text() == ftserieRef.text())
        self.assertTrue(dkrfRef.text() == self._ftserie.recons_params.dkrf["DKFILE"])
        ftserieRef.setText(txt + "tata")
        ftserieRef.editingFinished.emit()
        self.assertTrue(dkrfRef.text() == ftserieRef.text())
        self.assertTrue(dkrfRef.text() == self._ftserie.recons_params.dkrf["DKFILE"])

        # test what ref
        ftserieWhatDark = ftserieEditor._dkRefWidget._qcbDKMode
        dkrfWhatDark = self._darkRefCopy.mainWidget.tabGeneral._darkWCB
        self.assertTrue(ftserieWhatDark.getMode() == dkrfWhatDark.getMode())
        for mode in DkrfMethod:
            ftserieWhatDark.setMode(mode)
            self.assertTrue(dkrfWhatDark.getMode() == mode)
            self.assertTrue(self._ftserie.recons_params.dkrf["DARKCAL"] == mode)
        for mode in DkrfMethod:
            dkrfWhatDark.setMode(mode)
            self.assertTrue(ftserieWhatDark.getMode() == mode)
            self.assertTrue(self._ftserie.recons_params.dkrf["DARKCAL"] == mode)


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestReconsParamFtSerieDarkRef,):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "05/04/2019"


import unittest
import tempfile
import shutil
import os
from tomwer.core.utils.scanutils import MockEDF
from tomwer.core.scan.scanbase import TomwerScanBase
from tomwer.core.process.control.datatransfert import FolderTransfert
from tomwer.test.utils import UtilsTest
from nxtomomill import converter as nxtomomill_converter
from tomwer.core.scan.hdf5scan import HDF5TomoScan


class TestDataTransferIO(unittest.TestCase):
    """Test inputs and outputs types of the handler functions"""

    def setUp(self):
        self.origin_folder = tempfile.mkdtemp()
        self.scan_folder = os.path.join(self.origin_folder, "scan_toto")
        os.makedirs(self.scan_folder)
        self.output_folder = tempfile.mkdtemp()

        self.scan = MockEDF.mockScan(
            scanID=self.scan_folder, nRadio=10, nRecons=1, nPagRecons=4, dim=10
        )
        self.transfert_process = FolderTransfert()
        self.transfert_process.setDestDir(self.output_folder)

    def tearDown(self):
        shutil.rmtree(self.origin_folder)
        shutil.rmtree(self.output_folder)

    def testInputOutput(self):
        """Test that io using TomoBase instance work"""
        for input_type in (dict, TomwerScanBase):
            for _input in FolderTransfert.inputs:
                for return_dict in (True, False):
                    if os.path.exists(self.output_folder):
                        shutil.rmtree(self.output_folder)
                        os.makedirs(self.output_folder)

                    self.scan = MockEDF.mockScan(
                        scanID=self.scan_folder,
                        nRadio=10,
                        nRecons=1,
                        nPagRecons=4,
                        dim=10,
                    )

                    with self.subTest(
                        handler=_input.handler,
                        return_dict=return_dict,
                        input_type=input_type,
                    ):
                        input_obj = self.scan
                        if input_obj is dict:
                            input_obj = input_obj.to_dict()
                        self.transfert_process._set_return_dict(return_dict)
                        out = getattr(self.transfert_process, _input.handler)(input_obj)
                        if return_dict:
                            self.assertTrue(isinstance(out, dict))
                        else:
                            self.assertTrue(isinstance(out, TomwerScanBase))


class TestBlissDataTransfer(unittest.TestCase):
    """Make sure we can transfer data from bliss acquisition"""

    def setUp(self):
        self.input_dir = tempfile.mkdtemp()
        self.output_dir = tempfile.mkdtemp()
        shutil.copytree(
            UtilsTest.getBlissDataset(folderID="sample"),
            os.path.join(self.input_dir, "sample"),
        )

        self._proposal_file = os.path.join(
            self.input_dir, "sample", "ihpayno_sample.h5"
        )
        self._sample_file = os.path.join(
            self.input_dir, "sample", "sample_29042021", "sample_29042021.h5"
        )
        self._sample_file_entry = "1.1"

        self.process = FolderTransfert()
        self.process.setDestDir(self.output_dir)

        # convert it to nx
        output_file_path = os.path.join(
            self.input_dir, "sample", "sample_29042021", "test_nx.h5"
        )
        entries = ("/1.1",)
        assert os.path.exists(self._sample_file)
        conv = nxtomomill_converter.h5_to_nx(
            input_file_path=self._sample_file,
            output_file=output_file_path,
            entries=entries,
            single_file=False,
            ask_before_overwrite=False,
            request_input=False,
            file_extension=".nx",
        )
        assert len(conv) == 1
        conv_file, conv_entry = conv[0]
        self.scan = HDF5TomoScan(scan=conv_file, entry=conv_entry)

        # mock data listener: the processing of the Data transfer requires
        # knowledge of bliss files origin.
        from tomwer.core.process.control.datalistener import DataListener

        data_listener = DataListener()
        data_listener.process_sample_file(
            sample_file=self._sample_file,
            entry=self._sample_file_entry,
            proposal_file=self._proposal_file,
            master_sample_file=None,
        )

    def tearDown(self):
        shutil.rmtree(self.input_dir)
        shutil.rmtree(self.output_dir)

    def testDataTransfer(self):
        """Make sure the data transfer is able to retrieve the scan,
        proposal file and scan file to transfer.
        Check that only the specific scan folders will be copy and removed
        and the other won't be affected.
        """
        out_nx = os.path.join(
            self.output_dir, "sample", "sample_29042021", "test_nx.h5"
        )
        self.assertFalse(os.path.exists(out_nx))
        out_proposal = os.path.join(self.output_dir, "sample", "ihpayno_sample")
        self.assertFalse(os.path.exists(out_proposal))
        out_sample_file = os.path.join(
            self.output_dir, "sample", "sample_29042021", "sample_29042021.h5"
        )
        self.assertFalse(os.path.exists(out_sample_file))
        out_included_scans = [
            os.path.join(self.output_dir, "sample", "sample_29042021", "scan0002")
        ]
        for scan_path in out_included_scans:
            self.assertFalse(os.path.exists(scan_path))
        out_not_included_scans = [
            os.path.join(self.output_dir, "sample", "sample_29042021", "scan0004"),
            os.path.join(self.output_dir, "sample", "sample_29042021", "scan0006"),
            os.path.join(self.output_dir, "sample", "sample_29042021", "scan0008"),
        ]
        for scan_path in out_not_included_scans:
            self.assertFalse(os.path.exists(scan_path))

        self.process.process(scan=self.scan)

        self.assertTrue(os.path.exists(out_nx))
        self.assertTrue(os.path.exists(out_proposal))
        self.assertTrue(os.path.exists(out_sample_file))
        for scan_path in out_included_scans:
            self.assertTrue(os.path.exists(scan_path))
        for scan_path in out_not_included_scans:
            self.assertFalse(os.path.exists(scan_path))


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestDataTransferIO, TestBlissDataTransfer):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017-2019 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "27/02/2019"


from tomwer.gui.reconstruction.axis import AxisWindow
from tomwer.test.utils import skip_gui_test
from tomwer.test.utils import UtilsTest
from tomwer.core.process.reconstruction.axis.mode import AxisMode
from tomwer.core.process.reconstruction.axis.params import AxisCalculationInput
from tomwer.core.process.reconstruction.axis.projectiontype import ProjectionType
from tomwer.core.scan.edfscan import EDFTomoScan
from tomwer.synctools.axis import QAxisRP
from silx.gui.utils.testutils import TestCaseQt
from tomwer.core.scan.hdf5scan import HDF5TomoScan
from tomoscan.scanbase import _FOV
from silx.gui import qt
import numpy.random
import unittest
import tempfile
import os
import shutil
import h5py


@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestWindowAxis(TestCaseQt):
    """Test that the axis widget work correctly"""

    def setUp(self):
        self.recons_params = QAxisRP()
        self._window = AxisWindow(axis_params=self.recons_params)
        self.scan_path = UtilsTest.getEDFDataset("D2_H2_T2_h_")
        self.scan = EDFTomoScan(self.scan_path)
        self._window.show()
        self.qWaitForWindowExposed(self._window)

    def tearDown(self):
        self._window.setAttribute(qt.Qt.WA_DeleteOnClose)
        self._window.close()

    def testSetImagesNumpyArray(self):
        """Test the setImages function"""
        radio_axis = self._window._axisWidget._radioAxis
        self.assertEqual(radio_axis.getPlot().getPlot().getActiveImage(), None)
        imgA = numpy.random.random((100, 100))
        imgB = numpy.random.random((100, 100))
        radio_axis.setImages(imgA=imgA, imgB=imgB)
        self.qapp.processEvents()
        self.assertNotEqual(radio_axis.getPlot().getPlot().getActiveImage(), None)

    def testSetScan(self):
        """Test the setScan function"""
        radio_axis_plot = self._window._axisWidget._radioAxis.getPlot()
        self.assertEqual(radio_axis_plot.getPlot().getActiveImage(), None)
        self._window.setScan(self.scan)
        self.assertNotEqual(radio_axis_plot.getPlot().getActiveImage(), None)

    def testShiftButtons(self):
        """Test that the 'left', 'right', ... buttons and the shift steps are
        correctly working"""
        imgA = numpy.random.random((100, 100))
        imgB = numpy.random.random((100, 100))
        radio_axis = self._window._axisWidget._radioAxis
        radio_axis.setImages(imgA=imgA, imgB=imgB)
        self.qapp.processEvents()
        self.assertTrue(radio_axis.getXShift() == 0)
        self.assertTrue(radio_axis.getYShift() == 0)
        self.assertTrue(radio_axis.getShiftStep() == 1.0)
        radio_axis._controlWidget._shiftControl._leftButton.click()
        self.qapp.processEvents()
        self.assertTrue(radio_axis.getXShift() == -1)
        self.assertTrue(radio_axis.getYShift() == 0)
        radio_axis._controlWidget._shiftControl._rightButton.click()
        self.qapp.processEvents()
        self.assertTrue(radio_axis.getXShift() == 0)
        self.assertTrue(radio_axis.getYShift() == 0)
        radio_axis._controlWidget._displacementSelector._fineButton.toggle()
        self.qapp.processEvents()
        self.assertTrue(radio_axis.getShiftStep() == 0.1)
        radio_axis._controlWidget._shiftControl._topButton.click()
        self.qapp.processEvents()
        self.assertTrue(radio_axis.getXShift() == 0)
        self.assertTrue(radio_axis.getYShift() == 0.1)
        radio_axis.setShiftStep(0.2)
        self.assertTrue(radio_axis.getShiftStep() == 0.2)
        radio_axis._controlWidget._shiftControl._bottomButton.click()
        self.qapp.processEvents()
        self.assertTrue(radio_axis.getXShift() == 0)
        self.assertTrue(radio_axis.getYShift() == -0.1)

    def testAxisObjectGlobal(self):
        """Test that the GUI change according to the Axis object"""
        axis_obj = QAxisRP()
        self._window.setReconsParams(axis_obj)
        self.qapp.processEvents()
        self.assertEqual(
            self._window._axisWidget._radioAxis.getMode(), AxisMode.centered
        )
        # TODO: change the mode to see if the object is modify

    def testAxisObjectManual(self):
        """Test that the GUI change according to the Axis object"""
        axis_params = QAxisRP()
        axis_params.mode = AxisMode.manual
        axis_params.set_value_ref_tomwer(-6.0)
        # note : for now only the xshift is managed !
        self._window.setReconsParams(axis_params)
        radio_axis = self._window._axisWidget._radioAxis
        self.assertEqual(radio_axis.getMode(), AxisMode.manual)
        self.assertEqual(radio_axis.getXShift(), -6.0)
        self.assertEqual(radio_axis.getYShift(), 0.0)

    def testAxisCalculationUpdate(self):
        """Test that the ReconsParam object and the gui are synchronized"""
        self.recons_params = QAxisRP()
        self._window = AxisWindow(axis_params=self.recons_params)
        main_opt_widget = self._window._axisWidget._radioAxis._controlWidget._mainWidget
        self.assertTrue(
            self.recons_params.projection_type is ProjectionType.transmission
        )
        self.assertTrue(self.recons_params.paganin_preproc is False)
        self.assertTrue(
            main_opt_widget._optionsWidget._qcbDataMode.currentData(qt.Qt.UserRole)
            is AxisCalculationInput.transmission
        )
        with self.assertRaises(AttributeError):
            self.recons_params.axis.paganin_preproc = False
        with self.assertRaises(AttributeError):
            self.recons_params.axis.projection_type = ProjectionType.transmission

        # test that interface will synchronize with reconsparams
        self.recons_params.calculation_input_type = AxisCalculationInput.emission
        self.qapp.processEvents()
        self.assertTrue(
            main_opt_widget._optionsWidget._qcbDataMode.currentData(qt.Qt.UserRole)
            is AxisCalculationInput.emission
        )
        # test that reconsparams will synchronize with interface
        cal_type = AxisCalculationInput.transmission_pag.name()
        index = main_opt_widget._optionsWidget._qcbDataMode.findText(cal_type)
        assert index >= 0
        main_opt_widget._optionsWidget._qcbDataMode.setCurrentIndex(index)
        self.assertTrue(
            self.recons_params.calculation_input_type
            is AxisCalculationInput.transmission_pag
        )

    def testAxisCalculationSaveLoad(self):
        pass


@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestWindowsAxisSetScan(TestCaseQt):
    """Test that updates are correctly done in the case scan has some
    knowledge on"""

    def setUp(self):
        TestCaseQt.setUp(self)
        self.recons_params = QAxisRP()
        self.widget = AxisWindow(axis_params=self.recons_params)
        self.scan_dir = tempfile.mkdtemp()
        # create dataset
        self.master_file = os.path.join(self.scan_dir, "frm_edftomomill_twoentries.nx")
        shutil.copyfile(
            UtilsTest.getH5Dataset(folderID="frm_edftomomill_twoentries.nx"),
            self.master_file,
        )
        self.scan = HDF5TomoScan(scan=self.master_file, entry="entry0000")

    def tearDown(self):
        self.widget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.widget.close()
        self.widget = None
        TestCaseQt.tearDown(self)

    def patch_fov(self, value: str):
        with h5py.File(self.scan.master_file, mode="a") as h5s:
            for entry in ("entry0000", "entry0001"):
                entry_node = h5s[entry]
                if "instrument/detector/field_of_view" in entry_node:
                    del entry_node["instrument/detector/field_of_view"]
                entry_node["instrument/detector/field_of_view"] = value

    def testAlgorithmUpdate(self):
        """Make sure the algorithm is updated according to the field of view"""
        self.assertEqual(self.widget.getSelection(), "radio")
        self.assertEqual(self.widget.getRadioMode(), AxisMode.centered)
        self.patch_fov(value=_FOV.HALF.value)
        self.widget.setScan(self.scan)
        self.assertEqual(self.widget.getRadioMode(), AxisMode.global_)
        self.patch_fov(value=_FOV.FULL.value)
        self.scan.clear_caches()
        self.widget.setScan(self.scan)
        self.assertEqual(self.widget.getRadioMode(), AxisMode.global_)


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestWindowAxis, TestWindowsAxisSetScan):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""
contains gui relative to axis calculation using sinogram
"""

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "25/05/2020"


from contextlib import ContextDecorator


class _RadioSinoLocker(ContextDecorator):
    """Simple cntextManager for the radio and sinogram axis synchronization"""

    def __init__(self, parent):
        self.__parent = parent
        self.old_radio = None
        self.old_sinogram = None
        self.__locked = False

    def __enter__(self):
        self.old_radio = self.radioAxis.blockSignals(True)
        self.old_sinogram = self.sinogramAxis.blockSignals(True)

    @property
    def radioAxis(self):
        return self.__parent._radioAxis

    @property
    def sinogramAxis(self):
        return self.__parent._sinogramAxis

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.radioAxis.blockSignals(self.old_radio)
        self.sinogramAxis.blockSignals(self.old_sinogram)

    def isLocked(self):
        return self.__locked

    def setLocked(self, locked):
        if locked == self.__locked:
            return
        else:
            self.__locked = locked
            # block widgets signals
            with self:
                self.radioAxis.setLocked(locked)
                self.sinogramAxis.setLocked(locked)



"""Recons param module is containing classes used by reconstruction process
to configure them."""
from .reconsparams import ReconsParams
from .axis import AxisRP
from .beamgeo import BeamGeoRP
from .ft import FTRP
from .paganin import PaganinRP
from .pyhst import PyhstRP
